@@ -1,8 +1,8 @@
 package File::DataClass;
 
 use 5.010001;
-use namespace::sweep;
-use version; our $VERSION = qv( sprintf '0.41.%d', q$Rev: 1 $ =~ /\d+/gmx );
+use namespace::autoclean;
+use version; our $VERSION = qv( sprintf '0.42.%d', q$Rev: 1 $ =~ /\d+/gmx );
 
 use Moo;
 
@@ -24,7 +24,7 @@ File::DataClass - Structured data file IO with OO paradigm
 
 =head1 Version
 
-This document describes version v0.41.$Rev: 1 $ of L<File::DataClass>
+This document describes version v0.42.$Rev: 1 $ of L<File::DataClass>
 
 =head1 Synopsis
 
